/***********************************************************************************************************************
 *
 * OpenBlueSky - NetBeans Platform Enhancements
 * Copyright (C) 2006-2011 by Tidalwave s.a.s. (http://www.tidalwave.it)
 *
 ***********************************************************************************************************************
 *
 * Licensed under the Apache License, Version 2.0 (the "License"); you may not use this file except in compliance with
 * the License. You may obtain a copy of the License at
 *
 *     http://www.apache.org/licenses/LICENSE-2.0
 *
 * Unless required by applicable law or agreed to in writing, software distributed under the License is distributed on
 * an "AS IS" BASIS, WITHOUT WARRANTIES OR CONDITIONS OF ANY KIND, either express or implied.  See the License for the
 * specific language governing permissions and limitations under the License.
 *
 ***********************************************************************************************************************
 *
 * WWW: http://openbluesky.java.net
 * SCM: https://bitbucket.org/tidalwave/openbluesky-src
 *
 **********************************************************************************************************************/
package it.tidalwave.netbeans.util;

import javax.annotation.Nonnull;
import java.lang.reflect.Method;
import java.util.Comparator;
import java.util.logging.Logger;
import java.text.Collator;
import java.io.Serializable;

/*******************************************************************************
 *
 * @author  Fabrizio Giudici
 * @version $Id$
 *
 ******************************************************************************/
public class DisplayNameComparator<T> implements Comparator<T>, Serializable
  {
    private final static String CLASS = DisplayNameComparator.class.getName();
    private final static Logger logger = Logger.getLogger(CLASS);
    
    private final Method getDisplayNameMethod;
    
    private final Collator collator = Collator.getInstance();
    
    public DisplayNameComparator (final @Nonnull Class<T> clazz)
      {
        try 
          {
            getDisplayNameMethod = clazz.getMethod("getDisplayName");
          } 
        catch (NoSuchMethodException e) 
          {
            throw new RuntimeException(e);
          } 
        catch (SecurityException e) 
          {
            throw new RuntimeException(e);
          }
      }

    @Override
    public int compare (final @Nonnull T object1, final @Nonnull T object2)
      {
        try 
          {
            return collator.compare((String)getDisplayNameMethod.invoke(object1),
                                    (String)getDisplayNameMethod.invoke(object2));
          }
        catch (RuntimeException e)
          {
            throw e;
          }
        catch (Exception e) 
          {
            logger.severe("Can't run Collator on " + object1 + ", " + object2 + ": " + e);
            logger.throwing(CLASS, "compare()", e);
            throw new RuntimeException(e);
          }   
      }
  }
